@@ -1,5 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
+ * Copyright (C) 2004 Imendio AB
  * Copyright (C) 2001-2003 CodeFactory AB
  * Copyright (C) 2001-2003 Richard Hult <richard@imendio.com>
  * Copyright (C) 2001-2002 Mikael Hallendal <micke@imendio.com>
@@ -26,7 +27,7 @@
 #include <glib.h>
 #include <gtk/gtktreemodel.h>
 #include <gtk/gtktreednd.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libplanner/mrp-task.h>
 #include "planner-marshal.h"
 #include "planner-gantt-model.h"
@@ -37,29 +38,49 @@ enum {
 	LAST_SIGNAL
 };
 
-struct _MgGanttModelPriv {
+struct _PlannerGanttModelPriv {
 	MrpProject *project;
-	GHashTable *task2node_hash; /* Task -> Node mapping. */
+	GHashTable *task2node;
 	GNode      *tree;
+
+	GHashTable *task2cache;
+	gint        wbs_stamp;
 };
 
-static void      gantt_model_init             (MgGanttModel           *model);
-static void      gantt_model_class_init       (MgGanttModelClass      *class);
-static void      gantt_model_tree_model_init  (GtkTreeModelIface      *iface);
-#ifdef DND
-static void      gantt_model_drag_source_init (GtkTreeDragSourceIface *iface);
-static void      gantt_model_drag_dest_init   (GtkTreeDragDestIface   *iface);
-#endif
-static gboolean  gantt_model_get_iter         (GtkTreeModel           *model,
-					       GtkTreeIter            *iter,
-					       GtkTreePath            *path);
-static void      gantt_model_task_notify_cb   (MrpTask                *task,
-					       GParamSpec             *pspec,
-					       MgGanttModel           *model);
-static GtkTreePath *
-gantt_model_get_path_from_node                (MgGanttModel           *model,
-					       GNode                  *node);
-static void      dump_tree                    (GNode                  *node);
+typedef struct {
+	gchar *wbs;
+	gint   wbs_stamp;
+	
+	gchar *start;
+	gchar *finish;
+	gchar *duration;
+	gchar *work;
+} ValueCache;
+
+
+static void         gantt_model_init                 (PlannerGanttModel      *model);
+static void         gantt_model_class_init           (PlannerGanttModelClass *class);
+static void         gantt_model_tree_model_init      (GtkTreeModelIface      *iface);
+static gboolean     gantt_model_get_iter             (GtkTreeModel           *model,
+						      GtkTreeIter            *iter,
+						      GtkTreePath            *path);
+static void         gantt_model_task_notify_cb       (MrpTask                *task,
+						      GParamSpec             *pspec,
+						      PlannerGanttModel      *model);
+static void         gantt_model_task_prop_changed_cb (MrpTask                *task,
+						      MrpProperty            *property,
+						      GValue                 *value,
+						      PlannerGanttModel      *model);
+static GtkTreePath *gantt_model_get_path_from_node   (PlannerGanttModel      *model,
+						      GNode                  *node);
+static const gchar *value_cache_get_wbs              (PlannerGanttModel      *model,
+						      MrpTask                *task);
+static ValueCache * value_cache_get                  (PlannerGanttModel      *model,
+						      MrpTask                *task);
+static void         value_cache_clear                (PlannerGanttModel      *model,
+						      MrpTask                *task);
+static void         value_cache_clear_cache_wbs      (PlannerGanttModel      *model);
+static void         value_cache_free                 (ValueCache             *cache);
 
 
 static GObjectClass *parent_class;
@@ -72,13 +93,13 @@ planner_gantt_model_get_type (void)
 
 	if (!type) {
 		static const GTypeInfo info = {
-			sizeof (MgGanttModelClass),
+			sizeof (PlannerGanttModelClass),
 			NULL,		/* base_init */
 			NULL,		/* base_finalize */
 			(GClassInitFunc) gantt_model_class_init,
 			NULL,		/* class_finalize */
 			NULL,		/* class_data */
-			sizeof (MgGanttModel),
+			sizeof (PlannerGanttModel),
 			0,
 			(GInstanceInitFunc) gantt_model_init
 		};
@@ -89,55 +110,38 @@ planner_gantt_model_get_type (void)
 			NULL
 		};
 
-#ifdef DND
-		static const GInterfaceInfo drag_source_info = {
-			(GInterfaceInitFunc) gantt_model_drag_source_init,
-			NULL,
-			NULL
-		};
-
-		static const GInterfaceInfo drag_dest_info = {
-			(GInterfaceInitFunc) gantt_model_drag_dest_init,
-			NULL,
-			NULL
-		};
-#endif    
 		type = g_type_register_static (G_TYPE_OBJECT,
-					       "MgGanttModel",
+					       "PlannerGanttModel",
 					       &info, 0);
 		
 		g_type_add_interface_static (type,
 					     GTK_TYPE_TREE_MODEL,
 					     &tree_model_info);
-
-#ifdef DND
-		g_type_add_interface_static (type,
-					     GTK_TYPE_TREE_DRAG_SOURCE,
-					     &drag_source_info);
-
-		g_type_add_interface_static (type,
-					     GTK_TYPE_TREE_DRAG_DEST,
-					     &drag_dest_info);
-#endif
    	}
 
 	return type;
 }
 
 static void
-gantt_model_connect_to_task_signals (MgGanttModel *model, MrpTask *task)
+gantt_model_connect_to_task_signals (PlannerGanttModel *model, MrpTask *task)
 {
 	g_signal_connect_object (task,
 				 "notify",
 				 G_CALLBACK (gantt_model_task_notify_cb),
 				 model,
 				 0);
+
+	g_signal_connect_object (task,
+				 "prop_changed",
+				 G_CALLBACK (gantt_model_task_prop_changed_cb),
+				 model,
+				 0);
 }
 
 static void
-gantt_model_task_inserted_cb (MrpProject   *project,
-			      MrpTask      *task,
-			      MgGanttModel *model)
+gantt_model_task_inserted_cb (MrpProject        *project,
+			      MrpTask           *task,
+			      PlannerGanttModel *model)
 {
 	GtkTreePath *path;
 	GtkTreePath *parent_path;
@@ -148,14 +152,16 @@ gantt_model_task_inserted_cb (MrpProject   *project,
 	gint         pos;
 	gboolean     has_child_toggled;
 
+	value_cache_clear_cache_wbs (model);
+	
 	node = g_node_new (task);
 
-	g_hash_table_insert (model->priv->task2node_hash, task, node);
+	g_hash_table_insert (model->priv->task2node, task, node);
 
 	parent = mrp_task_get_parent (task);
 	pos = mrp_task_get_position (task);
 
-	parent_node = g_hash_table_lookup (model->priv->task2node_hash, parent);
+	parent_node = g_hash_table_lookup (model->priv->task2node, parent);
 
 	has_child_toggled = (g_node_n_children (parent_node) == 0);
 
@@ -179,30 +185,35 @@ gantt_model_task_inserted_cb (MrpProject   *project,
 	gantt_model_connect_to_task_signals (model, task);
 
 	/* Sanity check. */
-	if (g_node_n_nodes (model->priv->tree, G_TRAVERSE_ALL)
-	    != g_hash_table_size (model->priv->task2node_hash)) {
+#if 0
+	if (g_node_n_nodes (model->priv->tree, G_TRAVERSE_ALL) !=
+	    g_hash_table_size (model->priv->task2node)) {
 		g_warning ("Gantt model corrupt.");
 	}
-
+#endif
+	
 	g_signal_emit (model, signals[TASK_ADDED], 0, task);
 }
 
 static gboolean
-traverse_remove_subtree (GNode        *node,
-			 MgGanttModel *model)
+traverse_remove_subtree (GNode             *node,
+			 PlannerGanttModel *model)
 {
 	g_signal_handlers_disconnect_by_func (node->data,
 					      gantt_model_task_notify_cb,
 					      model);
+	g_signal_handlers_disconnect_by_func (node->data,
+					      gantt_model_task_prop_changed_cb,
+					      model);
 	
-	g_hash_table_remove (model->priv->task2node_hash, node->data);
+	g_hash_table_remove (model->priv->task2node, node->data);
 
 	return FALSE;
 }
 
 static void
-gantt_model_remove_subtree (MgGanttModel *model,
-			    GNode        *node)
+gantt_model_remove_subtree (PlannerGanttModel *model,
+			    GNode             *node)
 {	
 	g_node_unlink (node);
 
@@ -217,9 +228,9 @@ gantt_model_remove_subtree (MgGanttModel *model,
 }
 
 static void
-gantt_model_task_removed_cb (MrpProject   *project,
-			     MrpTask      *task,
-			     MgGanttModel *model)
+gantt_model_task_removed_cb (MrpProject        *project,
+			     MrpTask           *task,
+			     PlannerGanttModel *model)
 {
 	GNode       *node;
 	GNode       *parent_node;
@@ -229,18 +240,23 @@ gantt_model_task_removed_cb (MrpProject   *project,
 	gboolean     has_child_toggled;
 
 	g_signal_emit (model, signals[TASK_REMOVED], 0, task);
-	
-	node = g_hash_table_lookup (model->priv->task2node_hash, task);
+
+	node = g_hash_table_lookup (model->priv->task2node, task);
 	if (!node) {
 		/* This happens if we are already removed when a parent was
 		 * removed. 
 		 */
 		return; 
 	}
-	
+
+	value_cache_clear_cache_wbs (model);
+
 	g_signal_handlers_disconnect_by_func (task,
 					      gantt_model_task_notify_cb,
 					      model);
+	g_signal_handlers_disconnect_by_func (task,
+					      gantt_model_task_prop_changed_cb,
+					      model);
 	
 	parent_node = node->parent;
 	
@@ -265,57 +281,53 @@ gantt_model_task_removed_cb (MrpProject   *project,
 	gtk_tree_path_free (path);
 
 	/* Sanity check. */
-	if (g_node_n_nodes (model->priv->tree, G_TRAVERSE_ALL)
-	    != g_hash_table_size (model->priv->task2node_hash)) {
+#if 0
+	if (g_node_n_nodes (model->priv->tree, G_TRAVERSE_ALL) !=
+	    g_hash_table_size (model->priv->task2node)) {
 		g_warning ("Gantt model corrupt.");
 	}
-	
+#endif
 }
 
 static void
 gantt_model_reattach_subtasks (GtkTreeModel *tree_model,
 			       MrpTask      *task)
 {
-	MgGanttModel     *model;
-	MgGanttModelPriv *priv;
-	MrpTask          *child;
-	GNode            *node;
-	GNode            *parent_node;
-	GtkTreePath      *path;
-	GtkTreeIter       iter;
-	gint              pos;
-
-	model = MG_GANTT_MODEL (tree_model);
+	PlannerGanttModel     *model;
+	PlannerGanttModelPriv *priv;
+	MrpTask               *child;
+	GNode                 *node;
+	GNode                 *parent_node;
+	GtkTreePath           *path;
+	GtkTreeIter            iter;
+	gint                   pos;
+	gboolean               has_child_toggled;
+	
+	model = PLANNER_GANTT_MODEL (tree_model);
 	priv = model->priv;
 	
-	parent_node = g_hash_table_lookup (priv->task2node_hash, task);
+	parent_node = g_hash_table_lookup (priv->task2node, task);
 
 	/* Traverse the subtasks. */
 	child = mrp_task_get_first_child (task);
 	while (child) {
-		node = g_hash_table_lookup (priv->task2node_hash, child);
+		node = g_hash_table_lookup (priv->task2node, child);
 		pos = mrp_task_get_position (child);
 		g_node_insert (parent_node, pos, node);
 		
-#if 1
-		{
-			gboolean has_child_toggled;
-
-			has_child_toggled = (g_node_n_children (parent_node) == 1);
+		has_child_toggled = (g_node_n_children (parent_node) == 1);
+		
+		/* Emit has_child_toggled if necessary. */
+		if (has_child_toggled) {
+			GtkTreePath *parent_path;
 			
-			/* Emit has_child_toggled if necessary. */
-			if (has_child_toggled) {
-				GtkTreePath *parent_path;
-				
-				parent_path = gantt_model_get_path_from_node (model, parent_node);
-				gantt_model_get_iter (GTK_TREE_MODEL (model), &iter, parent_path);
-				gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (model),
-								      parent_path,
-								      &iter);
-				gtk_tree_path_free (parent_path);
-			}
+			parent_path = gantt_model_get_path_from_node (model, parent_node);
+			gantt_model_get_iter (GTK_TREE_MODEL (model), &iter, parent_path);
+			gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (model),
+							      parent_path,
+							      &iter);
+			gtk_tree_path_free (parent_path);
 		}
-#endif
 		
 		path = planner_gantt_model_get_path_from_task (model, child);
 		gtk_tree_model_get_iter (tree_model, &iter, path);
@@ -352,9 +364,9 @@ gantt_model_unlink_subtree_recursively (GNode *node)
 }
 
 static void
-gantt_model_task_moved_cb (MrpProject   *project,
-			   MrpTask      *task,
-			   MgGanttModel *model)
+gantt_model_task_moved_cb (MrpProject        *project,
+			   MrpTask           *task,
+			   PlannerGanttModel *model)
 {
 	MrpTask     *parent;
 	GtkTreePath *path;
@@ -365,6 +377,8 @@ gantt_model_task_moved_cb (MrpProject   *project,
 	gint         pos;
 	gboolean     has_child_toggled;
 
+	value_cache_clear_cache_wbs (model);
+
 	path = planner_gantt_model_get_path_from_task (model, task);
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
 
@@ -372,7 +386,7 @@ gantt_model_task_moved_cb (MrpProject   *project,
 	gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
 	gtk_tree_path_free (path);
 
-	node = g_hash_table_lookup (model->priv->task2node_hash, task);
+	node = g_hash_table_lookup (model->priv->task2node, task);
 
 	parent_node = node->parent;
 	has_child_toggled = (g_node_n_children (parent_node) == 1);
@@ -392,7 +406,7 @@ gantt_model_task_moved_cb (MrpProject   *project,
 
 	/* Get the new parent task and node. */
 	parent = mrp_task_get_parent (task);
-	parent_node = g_hash_table_lookup (model->priv->task2node_hash, parent);
+	parent_node = g_hash_table_lookup (model->priv->task2node, parent);
 
 	/* Re-insert the task at the new position. */
 	pos = mrp_task_get_position (task);
@@ -423,15 +437,22 @@ gantt_model_task_moved_cb (MrpProject   *project,
 }
 
 static void
-gantt_model_task_notify_cb (MrpTask      *task,
-			    GParamSpec   *pspec,
-			    MgGanttModel *model)
+gantt_model_task_notify_cb (MrpTask           *task,
+			    GParamSpec        *pspec,
+			    PlannerGanttModel *model)
 {
 	GtkTreeModel *tree_model;
 	GtkTreePath  *path;
 	GtkTreeIter   iter;
 
 	tree_model = GTK_TREE_MODEL (model);
+
+	if (strcmp (pspec->name, "start") == 0 ||
+	    strcmp (pspec->name, "finish") == 0 ||
+	    strcmp (pspec->name, "duration") == 0 ||
+	    strcmp (pspec->name, "work") == 0) {
+		value_cache_clear (model, task);
+	}
 	
 	path = planner_gantt_model_get_path_from_task (model, task);
 	gtk_tree_model_get_iter (tree_model, &iter, path);
@@ -440,11 +461,30 @@ gantt_model_task_notify_cb (MrpTask      *task,
 	gtk_tree_path_free (path);
 }
 
-static gboolean
-traverse_insert_task_into_hash (GNode        *node,
-				MgGanttModel *model)
+static void
+gantt_model_task_prop_changed_cb (MrpTask           *task,
+				  MrpProperty       *property,
+				  GValue            *value,
+				  PlannerGanttModel *model)
 {
-	g_hash_table_insert (model->priv->task2node_hash,
+	GtkTreeModel *tree_model;
+	GtkTreePath  *path;
+	GtkTreeIter   iter;
+
+	tree_model = GTK_TREE_MODEL (model);
+
+	path = planner_gantt_model_get_path_from_task (model, task);
+	gtk_tree_model_get_iter (tree_model, &iter, path);
+	gtk_tree_model_row_changed (tree_model, path, &iter);
+
+	gtk_tree_path_free (path);
+}
+
+static gboolean
+traverse_insert_task_into_hash (GNode             *node,
+				PlannerGanttModel *model)
+{
+	g_hash_table_insert (model->priv->task2node,
 			     node->data,
 			     node);
 
@@ -470,7 +510,7 @@ traverse_setup_tree (MrpTask  *task,
 }
 
 static GNode *
-gantt_model_setup_task_tree (MgGanttModel *model)
+gantt_model_setup_task_tree (PlannerGanttModel *model)
 {
 	MrpTask *root_task;
 	GNode   *root_node;
@@ -483,14 +523,14 @@ gantt_model_setup_task_tree (MgGanttModel *model)
 	return root_node;
 }
 
-MgGanttModel *
+PlannerGanttModel *
 planner_gantt_model_new (MrpProject *project)
 {
-	MgGanttModel     *model;
-	MgGanttModelPriv *priv;
-	GList            *tasks, *l;
+	PlannerGanttModel     *model;
+	PlannerGanttModelPriv *priv;
+	GList                 *tasks, *l;
 	
-	model = MG_GANTT_MODEL (g_object_new (MG_TYPE_GANTT_MODEL, NULL));
+	model = PLANNER_GANTT_MODEL (g_object_new (PLANNER_TYPE_GANTT_MODEL, NULL));
 	priv = model->priv;
 
 	priv->project = project;
@@ -534,10 +574,11 @@ planner_gantt_model_new (MrpProject *project)
 static void
 gantt_model_finalize (GObject *object)
 {
-	MgGanttModel *model = MG_GANTT_MODEL (object);
+	PlannerGanttModel *model = PLANNER_GANTT_MODEL (object);
 
 	g_node_destroy (model->priv->tree);
-	g_hash_table_destroy (model->priv->task2node_hash);
+	g_hash_table_destroy (model->priv->task2node);
+	g_hash_table_destroy (model->priv->task2cache);
 	
 	g_free (model->priv);
 	model->priv = NULL;
@@ -548,7 +589,7 @@ gantt_model_finalize (GObject *object)
 }
 
 static void
-gantt_model_class_init (MgGanttModelClass *klass)
+gantt_model_class_init (PlannerGanttModelClass *klass)
 {
 	GObjectClass *object_class;
 
@@ -589,6 +630,8 @@ gantt_model_get_column_type (GtkTreeModel *tree_model,
 			     gint          column)
 {
 	switch (column) {
+	case COL_WBS:
+		return G_TYPE_STRING;
 	case COL_NAME:
 		return G_TYPE_STRING;
 	case COL_START:
@@ -619,12 +662,12 @@ gantt_model_get_iter (GtkTreeModel *tree_model,
 		      GtkTreeIter  *iter,
 		      GtkTreePath  *path)
 {
-	MgGanttModel *gantt_model;
-	GtkTreeIter   parent;
-	gint         *indices;
-	gint          depth, i;
+	PlannerGanttModel *gantt_model;
+	GtkTreeIter        parent;
+	gint              *indices;
+	gint               depth, i;
 	
-	gantt_model = MG_GANTT_MODEL (tree_model);
+	gantt_model = PLANNER_GANTT_MODEL (tree_model);
 	
 	indices = gtk_tree_path_get_indices (path);
 	depth = gtk_tree_path_get_depth (path);
@@ -649,15 +692,15 @@ gantt_model_get_iter (GtkTreeModel *tree_model,
 }
 
 GtkTreePath *
-gantt_model_get_path_from_node (MgGanttModel *model,
-				GNode        *node)
+gantt_model_get_path_from_node (PlannerGanttModel *model,
+				GNode             *node)
 {
 	GtkTreePath *path;
 	GNode       *parent;
 	GNode       *child;
 	gint         i = 0;
 	
-	g_return_val_if_fail (MG_IS_GANTT_MODEL (model), NULL);
+	g_return_val_if_fail (PLANNER_IS_GANTT_MODEL (model), NULL);
 	g_return_val_if_fail (node != NULL, NULL);
 
 	parent = node->parent;
@@ -665,7 +708,7 @@ gantt_model_get_path_from_node (MgGanttModel *model,
 	if (parent == NULL && node == model->priv->tree) {
 		return gtk_tree_path_new_root ();
 	}
-	
+
 	g_assert (parent != NULL);
 
 	if (parent == model->priv->tree) {
@@ -705,18 +748,19 @@ gantt_model_get_path_from_node (MgGanttModel *model,
 }
 
 GtkTreePath *
-planner_gantt_model_get_path_from_task (MgGanttModel *model,
-				   MrpTask      *task)
+planner_gantt_model_get_path_from_task (PlannerGanttModel *model,
+					MrpTask           *task)
 {
 	GNode *node;
 	
-	g_return_val_if_fail (MG_IS_GANTT_MODEL (model), NULL);
+	g_return_val_if_fail (PLANNER_IS_GANTT_MODEL (model), NULL);
 	g_return_val_if_fail (MRP_IS_TASK (task), NULL);
 
-	node = g_hash_table_lookup (model->priv->task2node_hash, task);
+	node = g_hash_table_lookup (model->priv->task2node, task);
 
-	/* Help debugging. */
-	g_return_val_if_fail (node != NULL, NULL);
+	if (!node) {
+		return NULL;
+	}
 	
 	return gantt_model_get_path_from_node (model, node);
 }
@@ -729,11 +773,11 @@ gantt_model_get_path (GtkTreeModel *tree_model,
 	
 	g_return_val_if_fail (iter != NULL, NULL);
 	g_return_val_if_fail (iter->user_data != NULL, NULL);
-	g_return_val_if_fail (iter->stamp == MG_GANTT_MODEL (tree_model)->stamp, NULL);
+	g_return_val_if_fail (iter->stamp == PLANNER_GANTT_MODEL (tree_model)->stamp, NULL);
 
 	node = iter->user_data;
 
-	return gantt_model_get_path_from_node (MG_GANTT_MODEL (tree_model), node);
+	return gantt_model_get_path_from_node (PLANNER_GANTT_MODEL (tree_model), node);
 }
 
 static void
@@ -742,70 +786,69 @@ gantt_model_get_value (GtkTreeModel *tree_model,
 		       gint          column,
 		       GValue       *value)
 {
-	GNode      *node;
-	MrpTask    *task;
-	MrpProject *project;
-	gchar      *str;
-	mrptime     t, t1, t2;
-	gint        duration;
-	MrpTaskType type;
-
+	GNode       *node;
+	MrpTask     *task;
+	MrpProject  *project;
+	mrptime      t1, t2;
+	gint         duration;
+	MrpTaskType  type;
+	const gchar *name;
+	const gchar *cached_str;
+	
 	g_return_if_fail (iter != NULL);
 
 	node = iter->user_data;
 	task = node->data;
 
 	switch (column) {
+	case COL_WBS:
+		cached_str = value_cache_get_wbs (PLANNER_GANTT_MODEL (tree_model),
+						  task);
+		
+		g_value_init (value, G_TYPE_STRING);
+		g_value_set_string (value, cached_str);
+		break;
+		
 	case COL_NAME:
-		g_object_get (task, "name", &str, NULL);
-		if (str == NULL) {
-			str = g_strdup ("");
+		name = mrp_task_get_name (task);
+		if (name == NULL) {
+			name = "";
 		}
 		
 		g_value_init (value, G_TYPE_STRING);
-		g_value_set_string (value, str);
-
-		g_free (str);
+		g_value_set_string (value, name);
 		break;
 
 	case COL_START:
-		g_object_get (task, "start", &t, NULL);
-		
 		g_value_init (value, G_TYPE_LONG);
-		g_value_set_long (value, t);
+		g_value_set_long (value, mrp_task_get_work_start (task));
 		break;
 
 	case COL_FINISH:
-		g_object_get (task, "finish", &t, NULL);
-
 		g_value_init (value, G_TYPE_LONG);
-		g_value_set_long (value, t);
+		g_value_set_long (value, mrp_task_get_finish (task));
 		break;
 		
 	case COL_DURATION:
-		g_object_get (task, "duration", &duration, NULL);
-
 		g_value_init (value, G_TYPE_INT);
-		g_value_set_int (value, duration);
+		g_value_set_int (value, mrp_task_get_duration (task));
 		break;
 		
 	case COL_WORK:
-		g_object_get (task, "work", &duration, NULL);
-
 		g_value_init (value, G_TYPE_INT);
-		g_value_set_int (value, duration);
+		g_value_set_int (value, mrp_task_get_work (task));
 		break;
 
 	case COL_SLACK:
-		g_object_get (task,
-			      "finish", &t1,
-			      "latest-finish", &t2,
-			      "project", &project,
-			      NULL);
-
-		/* We don't support negative slack yet. */
+		t1 = mrp_task_get_finish (task);
+		t2 = mrp_task_get_latest_finish (task);
+		project = mrp_object_get_project (MRP_OBJECT (task));
+		
+		/* We don't support negative slack */
 		if (t2 >= t1) {
-			duration = mrp_project_calculate_task_work (
+			/* use summary duration calc since we want to use 
+			 * the project calendar and ignore resource assignments */
+			duration = mrp_project_calculate_summary_duration (
 				project, task, t1, t2);
 		} else {
 			duration = 0;
@@ -825,7 +868,7 @@ gantt_model_get_value (GtkTreeModel *tree_model,
 		break;
 
 	case COL_EDITABLE:
-		g_object_get (task, "type", &type, NULL);
+		type = mrp_task_get_task_type (task);
 		
 		g_value_init (value, G_TYPE_BOOLEAN);
 		if (g_node_n_children (node) > 0) {
@@ -884,7 +927,7 @@ gantt_model_iter_children (GtkTreeModel *tree_model,
 	if (parent) {
 		node = parent->user_data;
 	} else {
-		node = MG_GANTT_MODEL (tree_model)->priv->tree;
+		node = PLANNER_GANTT_MODEL (tree_model)->priv->tree;
 	}
 	
 	child = g_node_first_child (node);
@@ -895,7 +938,7 @@ gantt_model_iter_children (GtkTreeModel *tree_model,
 	}
 	
 	iter->user_data = child;
-	iter->stamp = MG_GANTT_MODEL (tree_model)->stamp;
+	iter->stamp = PLANNER_GANTT_MODEL (tree_model)->stamp;
 	return TRUE;
 }
 
@@ -919,7 +962,7 @@ gantt_model_iter_n_children (GtkTreeModel *tree_model,
 	if (iter) {
 		node = iter->user_data;
 	} else {
-		node = MG_GANTT_MODEL (tree_model)->priv->tree;
+		node = PLANNER_GANTT_MODEL (tree_model)->priv->tree;
 	}
 
 	return g_node_n_children (node);
@@ -931,13 +974,13 @@ gantt_model_iter_nth_child (GtkTreeModel *tree_model,
 			    GtkTreeIter  *parent_iter,
 			    gint          n)
 {
-	MgGanttModel *model;
+	PlannerGanttModel *model;
 	GNode        *parent;
 	GNode        *child;
 
 	g_return_val_if_fail (parent_iter == NULL || parent_iter->user_data != NULL, FALSE);
 
-	model = MG_GANTT_MODEL (tree_model);
+	model = PLANNER_GANTT_MODEL (tree_model);
 	
 	if (parent_iter == NULL) {
 		parent = model->priv->tree;
@@ -975,7 +1018,7 @@ gantt_model_iter_parent (GtkTreeModel *tree_model,
 	}
 
 	iter->user_data = node_parent;
-	iter->stamp = MG_GANTT_MODEL (tree_model)->stamp;
+	iter->stamp = PLANNER_GANTT_MODEL (tree_model)->stamp;
 
 	return TRUE;
 }
@@ -997,277 +1040,41 @@ gantt_model_tree_model_init (GtkTreeModelIface *iface)
 }
 
 static void
-gantt_model_init (MgGanttModel *model)
+gantt_model_init (PlannerGanttModel *model)
 {
-	MgGanttModelPriv *priv;
+	PlannerGanttModelPriv *priv;
 
-	priv = g_new0 (MgGanttModelPriv, 1);
+	priv = g_new0 (PlannerGanttModelPriv, 1);
 	model->priv = priv;
 
-	priv->task2node_hash = g_hash_table_new (NULL, NULL);
+	priv->task2node = g_hash_table_new (NULL, NULL);
+	priv->task2cache = g_hash_table_new_full (NULL, NULL,
+						  NULL,
+						  (GDestroyNotify) value_cache_free);
 	
 	do {
 		model->stamp = g_random_int ();
 	} while (model->stamp == 0);
 }
 
-/* DND */
-#ifdef DND
-static gboolean
-gantt_model_drag_data_delete (GtkTreeDragSource *drag_source,
-			      GtkTreePath       *path)
-{
-	GtkTreeIter iter;
-	
-	g_return_val_if_fail (MG_IS_GANTT_MODEL (drag_source), FALSE);
-
-	g_print ("delete\n");
-	
-	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (drag_source),
-				     &iter,
-				     path)) {
-		mrp_project_remove_task (MG_GANTT_MODEL (drag_source)->priv->project,
-					 iter.user_data);
-		return TRUE;
-	} else {
-		return FALSE;
-	}
-}
-
-static gboolean
-gantt_model_drag_data_get (GtkTreeDragSource *drag_source,
-			   GtkTreePath       *path,
-			   GtkSelectionData  *selection_data)
-{
-	g_return_val_if_fail (MG_IS_GANTT_MODEL (drag_source), FALSE);
-
-	/* Note that we don't need to handle the GTK_TREE_MODEL_ROW
-	 * target, because the default handler does it for us, but
-	 * we do anyway for the convenience of someone maybe overriding the
-	 * default handler.
-	 */
-	
-	if (gtk_tree_set_row_drag_data (selection_data,
-					GTK_TREE_MODEL (drag_source),
-					path)) {
-		return TRUE;
-	} else {
-		/* FIXME handle text targets at least. */
-	}
-
-	return FALSE;
-}
-
-static gboolean
-gantt_model_drag_data_received (GtkTreeDragDest   *drag_dest,
-				GtkTreePath       *dest,
-				GtkSelectionData  *selection_data)
-{
-	GtkTreeModel *tree_model;
-	MgGanttModel *gantt_model;
-	GtkTreeModel *src_model = NULL;
-	GtkTreePath *src_path = NULL;
-	gboolean retval = FALSE;
-
-	g_return_val_if_fail (MG_IS_GANTT_MODEL (drag_dest), FALSE);
-
-	g_print ("received\n");
-	
-	tree_model = GTK_TREE_MODEL (drag_dest);
-	gantt_model = MG_GANTT_MODEL (drag_dest);
-
-	if (gtk_tree_get_row_drag_data (selection_data,
-					&src_model,
-					&src_path) &&
-	    src_model == tree_model) {
-		/* Copy the given row to a new position */
-		GtkTreeIter src_iter;
-		GtkTreeIter dest_iter;
-		GtkTreePath *prev;
-		
-		if (!gtk_tree_model_get_iter (src_model,
-					      &src_iter,
-					      src_path)) {
-			goto out;
-		}
-
-		/* Get the path to insert _after_ (dest is the path to insert _before_) */
-		prev = gtk_tree_path_copy (dest);
-		
-		if (!gtk_tree_path_prev (prev)) {
-			GtkTreeIter dest_parent;
-			GtkTreePath *parent;
-			GtkTreeIter *dest_parent_p;
-			
-			/* dest was the first spot at the current depth; which means
-			 * we are supposed to prepend.
-			 */
-			
-			/* Get the parent, NULL if parent is the root */
-			dest_parent_p = NULL;
-			parent = gtk_tree_path_copy (dest);
-			if (gtk_tree_path_up (parent) &&
-			    gtk_tree_path_get_depth (parent) > 0) {
-				gtk_tree_model_get_iter (tree_model,
-							 &dest_parent,
-							 parent);
-				dest_parent_p = &dest_parent;
-			}
-			gtk_tree_path_free (parent);
-			parent = NULL;
-			
-			/*gtk_tree_store_prepend (GTK_TREE_STORE (tree_model),
-			  &dest_iter,
-			  dest_parent_p);*/
-			
-			retval = TRUE;
-		} else {
-			if (gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_model),
-						     &dest_iter,
-						     prev)) {
-				GtkTreeIter tmp_iter = dest_iter;
-				
-				if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tree_model),
-									"gtk-tree-model-drop-append"))) {
-					GtkTreeIter parent;
-					
-					if (gtk_tree_model_iter_parent (GTK_TREE_MODEL (tree_model), &parent, &tmp_iter))
-						/*gtk_tree_store_append (GTK_TREE_STORE (tree_model),
-						  &dest_iter, &parent);*/
-						;
-					else
-						/*gtk_tree_store_append (GTK_TREE_STORE (tree_model),
-						  &dest_iter, NULL);*/
-						;
-				}
-				else
-					/*gtk_tree_store_insert_after (GTK_TREE_STORE (tree_model),
-					  &dest_iter,
-					  NULL,
-					  &tmp_iter);*/
-					;
-				retval = TRUE;
-				
-			}
-		}
-		
-		g_object_set_data (G_OBJECT (tree_model),
-				   "gtk-tree-model-drop-append",
-				   NULL);
-		
-		gtk_tree_path_free (prev);
-		
-		/* If we succeeded in creating dest_iter, walk src_iter tree branch,
-		 * duplicating it below dest_iter.
-		 */
-
-		if (retval) {
-			/*recursive_node_copy (tree_store,
-			  &src_iter,
-			  &dest_iter);*/
-		}
-	} else {
-		/* FIXME maybe add some data targets eventually, or handle text
-		 * targets in the simple case.
-		 */
-	}
-	
- out:
-	
-	if (src_path) {
-		gtk_tree_path_free (src_path);
-	}
-	
-	return retval;
-}
-
-static gboolean
-gantt_model_row_drop_possible (GtkTreeDragDest  *drag_dest,
-			       GtkTreePath      *dest_path,
-			       GtkSelectionData *selection_data)
-{
-	GtkTreeModel *src_model = NULL;
-	GtkTreePath  *src_path = NULL;
-	GtkTreePath  *tmp = NULL;
-	gboolean      retval = FALSE;
-	GtkTreeIter   iter;
-	
-	if (!gtk_tree_get_row_drag_data (selection_data,
-					 &src_model,
-					 &src_path)) {
-		goto out;
-	}
-	
-	/* can only drag to ourselves */
-	if (src_model != GTK_TREE_MODEL (drag_dest)) {
-		goto out;
-	}
-	
-	/* Can't drop into ourself. */
-	if (gtk_tree_path_is_ancestor (src_path,
-				       dest_path)) {
-		goto out;
-	}
-
-	/* Can't drop if dest_path's parent doesn't exist */
-	if (gtk_tree_path_get_depth (dest_path) > 1) {
-		tmp = gtk_tree_path_copy (dest_path);
-		gtk_tree_path_up (tmp);
-		
-		if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (drag_dest),
-					      &iter, tmp)) {
-			goto out;
-		}
-	}
-
-	/* Can otherwise drop anywhere. */
-	retval = TRUE;
-
- out:
-	
-	if (src_path) {
-		gtk_tree_path_free (src_path);
-	}
-	if (tmp) {
-		gtk_tree_path_free (tmp);
-	}
-	
-	return retval;
-}
-
-static void
-gantt_model_drag_source_init (GtkTreeDragSourceIface *iface)
-{
-	iface->drag_data_delete = gantt_model_drag_data_delete;
-	iface->drag_data_get = gantt_model_drag_data_get;
-}
-
-static void
-gantt_model_drag_dest_init (GtkTreeDragDestIface *iface)
-{
-	iface->drag_data_received = gantt_model_drag_data_received;
-	iface->row_drop_possible = gantt_model_row_drop_possible;
-}
-#endif
-
 MrpProject *
-planner_gantt_model_get_project (MgGanttModel *model)
+planner_gantt_model_get_project (PlannerGanttModel *model)
 {
-	g_return_val_if_fail (MG_IS_GANTT_MODEL (model), NULL);
+	g_return_val_if_fail (PLANNER_IS_GANTT_MODEL (model), NULL);
 	
 	return model->priv->project;
 }
 
 MrpTask *
-planner_gantt_model_get_task (MgGanttModel *model,
-			 GtkTreeIter  *iter)
+planner_gantt_model_get_task (PlannerGanttModel *model,
+			      GtkTreeIter  *iter)
 {
 	MrpTask *task;
 
 	task = ((GNode *) iter->user_data)->data; 
 
 	if (task == NULL) {
-		g_warning ("Eeek");
+		/* Shouldn't really happen. */
 		return NULL;
 	} else { 
 		return MRP_TASK (task);
@@ -1275,16 +1082,34 @@ planner_gantt_model_get_task (MgGanttModel *model,
 }
 
 MrpTask *
-planner_gantt_model_get_indent_task_target (MgGanttModel *model,
-				       MrpTask      *task)
+planner_gantt_model_get_task_from_path (PlannerGanttModel *model,
+					GtkTreePath       *path)
+{
+	GtkTreeIter  iter;
+	MrpTask     *task = NULL;
+
+	g_return_val_if_fail (PLANNER_IS_GANTT_MODEL (model), NULL);
+	
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path)) {
+		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 
+				    COL_TASK, &task,
+				    -1);
+	}
+	
+	return task;
+}
+
+MrpTask *
+planner_gantt_model_get_indent_task_target (PlannerGanttModel *model,
+					    MrpTask           *task)
 {
 	GNode *node;
 	GNode *sibling;
 
-	g_return_val_if_fail (MG_IS_GANTT_MODEL (model), NULL);
+	g_return_val_if_fail (PLANNER_IS_GANTT_MODEL (model), NULL);
 	g_return_val_if_fail (MRP_IS_TASK (task), NULL);
 
-	node = g_hash_table_lookup (model->priv->task2node_hash, task);
+	node = g_hash_table_lookup (model->priv->task2node, task);
 
 	sibling = g_node_prev_sibling (node);
 
@@ -1296,65 +1121,99 @@ planner_gantt_model_get_indent_task_target (MgGanttModel *model,
 	return sibling->data;
 }
 
-
-
-/* ------------------ test */
-static gchar*
-get_n_chars (gint n, gchar c)
+static const gchar *
+value_cache_get_wbs (PlannerGanttModel *model,
+		     MrpTask           *task)
 {
-	GString *str;
-	gchar   *ret;
-	gint     i;
+	ValueCache *cache;
+	MrpTask    *tmp_task;
+	gchar      *str;
+	GString    *string;
+	gint        pos;
 
-	str = g_string_new ("");
+	cache = value_cache_get (model, task);
+	if (!cache->wbs) {
+		goto update_cache;
+	}
 	
-	for (i = 0; i < n; i++) {
-		g_string_append_c (str, c);
+	if (cache->wbs_stamp != model->priv->wbs_stamp) {
+		goto update_cache;
 	}
 
-	ret = str->str;
-	g_string_free (str, FALSE);
+	return cache->wbs;
 	
-	return ret;
-}
+ update_cache:
+	string = g_string_sized_new (24);
 
-static void
-dump_children (GNode *node, gint depth)
-{
-	GNode   *child;
-	gchar   *padding = get_n_chars (2 * depth, ' ');
-	MrpTask *task;
-	gchar   *name;
-
-	for (child = g_node_first_child (node); child; child = g_node_next_sibling (child)) {
-		task = (MrpTask *) child->data;
-
-		g_object_get (task, "name", &name, NULL);
-		g_print ("%sName: %s\n", padding, name);
-		g_free (name);
+	pos = -1;
+	tmp_task = task;
+	while (tmp_task) {
+		if (pos != -1) {
+			g_string_prepend_c (string, '.');
+		}
 		
-		dump_children (child, depth + 1);
+		pos = mrp_task_get_position (tmp_task) + 1;
+		
+		str = g_strdup_printf ("%d", pos);
+		g_string_prepend (string, str);
+		g_free (str);
+		
+		tmp_task = mrp_task_get_parent (tmp_task);
+		
+		/* Skip the root. */
+		if (mrp_task_get_parent (tmp_task) == NULL)
+			break;
 	}
 
-	g_free (padding);
+	g_free (cache->wbs);
+
+	cache->wbs = g_string_free (string, FALSE);
+	cache->wbs_stamp = model->priv->wbs_stamp;
+	
+	return cache->wbs;
 }
 
 static void
-dump_tree (GNode *node)
+value_cache_free (ValueCache *cache)
 {
-	g_return_if_fail (node != NULL);
-	g_return_if_fail (node->parent == NULL);
+	g_free (cache->wbs);
+	g_free (cache->start);
+	g_free (cache->finish);
+	g_free (cache->duration);
+	g_free (cache->work);
 
-	return;
-	
-	g_print ("------------------------------------------\n<Root>\n");
-
-	dump_children (node, 1);
-
-	g_print ("\n");
-
-	if (0) {
-		dump_tree (NULL);
-	}
+	g_free (cache);
 }
 
+static ValueCache *
+value_cache_get (PlannerGanttModel *model,
+		 MrpTask           *task)
+{
+	ValueCache *cache;
+
+	cache = g_hash_table_lookup (model->priv->task2cache, task);
+	if (!cache) {
+		cache = g_new0 (ValueCache, 1);
+		g_hash_table_insert (model->priv->task2cache,
+				     task,
+				     cache);
+	}	
+
+	return cache;
+}
+
+static void
+value_cache_clear (PlannerGanttModel *model,
+		   MrpTask           *task)
+{
+	g_hash_table_remove (model->priv->task2cache, task);
+}
+
+static void
+value_cache_clear_cache_wbs (PlannerGanttModel *model)
+{
+	model->priv->wbs_stamp++;
+}
+
+
+
